@@ -5,55 +5,91 @@ use strict;
 use warnings;
 #use Log::Any qw($log);
 
-our $VERSION = '0.13'; # VERSION
+our $VERSION = '0.14'; # VERSION
 
 require Exporter;
 our @ISA       = qw(Exporter);
-our @EXPORT_OK = qw(words2nums words2nums_simple);
-
-use vars qw(
-               %Digits
-               %Mults
-               %Words
-               $Neg_pat
-               $Exp_pat
-               $Dec_pat
-       );
+our @EXPORT_OK = qw(words2nums words2nums_simple $Pat);
 
 use Parse::Number::ID qw(parse_number_id);
 use Scalar::Util qw(looks_like_number);
 
-%Digits = (
-    nol => 0, kosong => 0,
-    se => 1, satu => 1,
+my %Digits = (
+    nol => 0, kosong => 0, ksg => 0, ksng => 0,
+    se => 1, satu => 1, st => 1,
     dua => 2,
-    tiga => 3,
-    empat => 4,
-    lima => 5,
-    enam => 6,
-    tujuh => 7,
-    delapan => 8,
-    sembilan => 9,
+    tiga => 3, tg => 3,
+    empat => 4, pat => 4, ampat => 4, mpat => 4,
+    lima => 5, lm => 5,
+    enam => 6, nam => 6,
+    tujuh => 7, tjh => 7,
+    delapan => 8, dlpn => 8, lapan => 8,
+    sembilan => 9, smbln => 9,
 );
 
-%Mults = (
-    puluh => 1e1,
-    ratus => 1e2,
-    ribu => 1e3,
-    juta => 1e6,
+my %Mults = (
+    puluh => 1e1, plh => 1e1,
+    ratus => 1e2, rts => 1e2,
+    ribu => 1e3, rb => 1e3,
+    juta => 1e6, jt => 1e6,
     milyar => 1e9, milyard => 1e9, miliar => 1e9, miliard => 1e9,
     triliun => 1e12, trilyun => 1e12,
+
+    # -yun / kw- / etc variants?
+    kuadriliun => 1e15,
+    kuintiliun => 18,
+    sekstiliun => 21,
+    septiliun => 24,
+    oktiliun => 27,
+    noniliun => 30,
+    desiliun => 33,
+    undesiliun => 36,
+    duodesiliun => 39,
+    tredesiliun => 42,
+    kuatuordesiliun => 45,
+    kuindesiliun => 48,
+    seksdesiliun => 51,
+    septendesiliun => 54,
+    oktodesiliun => 57,
+    novemdesiliun => 60,
+    vigintiliun => 63,
+    googol => 100, gugol => 100,
+    sentiliun => 303,
 );
 
-%Words = (
-    %Digits,
-    %Mults,
+my %Teen_Words = (
     belas => 0,
+    bls => 0,
 );
 
-$Neg_pat  = '(?:negatif|min|minus)';
-$Exp_pat  = '(?:(?:di)?kali(?:kan)? sepuluh pangkat)';
-$Dec_pat  = '(?:koma|titik)';
+my %Words = (
+    %Digits, %Mults, %Teen_Words,
+);
+
+my %Se = ("se" => 0, "s" => 0);
+
+# words that can be used with se- (or single digit), e.g. sebelas, tiga belas,
+# sepuluh, seratus, dua ratus, ...
+my %Se_Words = (
+    %Mults, %Teen_Words,
+);
+
+my $Neg_pat  = qr/(?:negatif|ngtf|min|minus|mns)/;
+my $Exp_pat  = qr/(?:(?:di)?\s*(?:kali|kl)(?:kan)?\s+(?:sepuluh|splh)
+                      \s+(?:pangkat|pkt|pngkt))/x;
+my $Dec_pat  = qr/(?:koma|km|titik|ttk)/;
+my $Teen_pat = "(?:".join("|", sort keys %Teen_Words).")";
+my $Mult_pat = "(?:" . join("|", sort keys %Se_Words).")";
+my $Se_pat   = "(?:" . join("|", sort keys %Se).")";
+my $Se_Mult_pat = "(?:(?:" . join("|", sort keys %Se).")".
+    "(?:" . join("|", sort keys %Se_Words) . "))";
+
+# quick pattern for extracting words ;
+my $_w = "(?:" . join("|", $Se_Mult_pat,
+                      (grep {$_ ne 'se'} sort keys(%Words)),
+                      $Parse::Number::ID::Pat,
+                  ) . ")";
+our $Pat = qr/(?:$_w(?:\s*$_w)*)/;
 
 sub words2nums($) { _handle_exp(@_) }
 sub words2nums_simple($) { _handle_simple(@_) }
@@ -143,7 +179,7 @@ sub _handle_int($) {
             $seen_digits = 1;
         }
 
-        elsif( $w eq 'belas' ) { # special case, teens (belasan)
+        elsif( $w =~ /^$Teen_pat$/ ) { # special case, teens (belasan)
             #$log->trace("saw a teen: $w");
             return undef unless $seen_digits; # mistake in writing teens
             push @nums, 10 + pop @nums;
@@ -215,10 +251,10 @@ sub _split_it($) {
             push @words, $n;
             push @words, $w2 if length($w2);
         }
-        elsif( $w =~ /^se(.+)$/ and defined $Words{$1} ) {
+        elsif( $w =~ /^($Se_pat)($Mult_pat)$/ and defined $Words{$1} ) {
             #$log->trace("i should split $w");
-            push @words, 'se', $1 }
-        elsif( $w =~ /^(.+)(belas|puluh|ratus|ribu|juta|mil[iy]ard?|tril[iy]un)$/ and defined $Words{$1} ) {
+            push @words, $1, $2 }
+        elsif( $w =~ /^(.+)\s+($Mult_pat)$/ and defined $Words{$1} ) {
             #$log->trace("i should split $w");
             push @words, $1, $2 }
         elsif( defined $Words{$w} ) {
@@ -235,18 +271,18 @@ sub _split_it($) {
 }
 
 1;
-# ABSTRACT: Convert Indonesian words (or mixture of numbers and words) to number
+# ABSTRACT: Convert Indonesian verbage to number
 
 
 =pod
 
 =head1 NAME
 
-Lingua::ID::Words2Nums - Convert Indonesian words (or mixture of numbers and words) to number
+Lingua::ID::Words2Nums - Convert Indonesian verbage to number
 
 =head1 VERSION
 
-version 0.13
+version 0.14
 
 =head1 SYNOPSIS
 
@@ -254,8 +290,8 @@ version 0.13
 
  print words2nums("seratus tiga koma dua");  # 103.2
  print words2nums("minus 3 juta 100 ribu");  # 3100000
- print words2nums("1,605 juta");             # 1605000
- print words2nums("-1,3e4");                 # 13000
+ print words2nums("1,605 jt");               # 1605000 (abbreviations accepted)
+ print words2nums("-1.3e4");                 # 13000
 
  print words2nums_simple("satu dua tiga");   # 123
 
@@ -265,6 +301,16 @@ This module provides two functions, B<words2nums> and B<words2nums_simple>. They
 are the counterpart of L<Lingua::ID::Nums2Words>'s B<nums2words> and
 B<nums2words_simple>.
 
+=head1 VARIABLES
+
+None are exported by default, but they are exportable.
+
+=head2 $Pat (REGEX)
+
+A regex for quickly matching/extracting verbage from text; it looks for a string
+of words. It's not perfect (the extracted verbage might not be valid, e.g. "ribu
+tiga"), but it's simple and fast.
+
 =head2 FUNCTIONS
 
 None are exported, but they are exportable.
@@ -287,6 +333,8 @@ Like B<words2nums>, but can only handle spelled digits (like "one two three" =>
 
 L<Lingua::ID::Nums2Words>
 
+L<Parse::Number::ID> is used to parse numbers in the verbage.
+
 =head1 AUTHOR
 
 Steven Haryanto <stevenharyanto@gmail.com>
